@@ -10,6 +10,7 @@ import re
 import stat
 import subprocess  # nosec: Needed for gitignore support.
 import sys
+from dataclasses import dataclass, field
 from functools import lru_cache
 from pathlib import Path
 from typing import (
@@ -30,7 +31,6 @@ from typing import (
 from warnings import warn
 
 from . import sorting, stdlibs
-from ._future import dataclass, field
 from .exceptions import (
     FormattingPluginDoesNotExist,
     InvalidSettingsPath,
@@ -38,7 +38,7 @@ from .exceptions import (
     SortingFunctionDoesNotExist,
     UnsupportedSettings,
 )
-from .profiles import profiles
+from .profiles import profiles as profiles
 from .sections import DEFAULT as SECTION_DEFAULTS
 from .sections import FIRSTPARTY, FUTURE, LOCALFOLDER, STDLIB, THIRDPARTY
 from .utils import Trie
@@ -50,11 +50,14 @@ if TYPE_CHECKING:
 else:
     from ._vendored import tomli
 
-_SHEBANG_RE = re.compile(br"^#!.*\bpython[23w]?\b")
+_SHEBANG_RE = re.compile(rb"^#!.*\bpython[23w]?\b")
 CYTHON_EXTENSIONS = frozenset({"pyx", "pxd"})
 SUPPORTED_EXTENSIONS = frozenset({"py", "pyi", *CYTHON_EXTENSIONS})
 BLOCKED_EXTENSIONS = frozenset({"pex"})
-FILE_SKIP_RE = re.compile(r"^#?\s?isort:\s?skip_file")
+FILE_SKIP_COMMENTS: Tuple[str, ...] = (
+    "isort:" + "skip_file",
+    "isort: " + "skip_file",
+)  # Concatenated to avoid this file being skipped
 MAX_CONFIG_SEARCH_DEPTH: int = 25  # The number of parent directories to for a config file within
 STOP_CONFIG_SEARCH_ON_DIRS: Tuple[str, ...] = (".git", ".hg")
 VALID_PY_TARGETS: Tuple[str, ...] = tuple(
@@ -235,10 +238,12 @@ class _Config:
     reverse_sort: bool = False
     star_first: bool = False
     import_dependencies = Dict[str, str]
-    git_ignore: Dict[Path, Set[Path]] = field(default_factory=dict)
+    git_ls_files: Dict[Path, Set[str]] = field(default_factory=dict)
     format_error: str = "{error}: {message}"
     format_success: str = "{success}: {message}"
     sort_order: str = "natural"
+    sort_reexports: bool = False
+    split_on_trailing_comma: bool = False
 
     def __post_init__(self) -> None:
         py_version = self.py_version
@@ -246,7 +251,7 @@ class _Config:
             if sys.version_info.major == 2 and sys.version_info.minor <= 6:
                 py_version = "2"
             elif sys.version_info.major == 3 and (
-                sys.version_info.minor <= 5 or sys.version_info.minor >= 10
+                sys.version_info.minor <= 5 or sys.version_info.minor >= 12
             ):
                 py_version = "3"
             else:
@@ -313,7 +318,7 @@ class Config(_Config):
             config_vars.pop("_skips")
             config_vars.pop("_skip_globs")
             config_vars.pop("_sorting_function")
-            super().__init__(**config_vars)  # type: ignore
+            super().__init__(**config_vars)
             return
 
         # We can't use self.quiet to conditionally show warnings before super.__init__() is called
@@ -455,13 +460,14 @@ class Config(_Config):
         if "src_paths" not in combined_config:
             combined_config["src_paths"] = (path_root / "src", path_root)
         else:
-            src_paths: Set[Path] = set()
+            src_paths: List[Path] = []
             for src_path in combined_config.get("src_paths", ()):
                 full_paths = (
                     path_root.glob(src_path) if "*" in str(src_path) else [path_root / src_path]
                 )
                 for path in full_paths:
-                    src_paths.add(path)
+                    if path not in src_paths:
+                        src_paths.append(path)
 
             combined_config["src_paths"] = tuple(src_paths)
 
@@ -519,7 +525,7 @@ class Config(_Config):
         if unsupported_config_errors:
             raise UnsupportedSettings(unsupported_config_errors)
 
-        super().__init__(sources=tuple(sources), **combined_config)  # type: ignore
+        super().__init__(sources=tuple(sources), **combined_config)
 
     def is_supported_filetype(self, file_name: str) -> bool:
         _root, ext = os.path.splitext(file_name)
@@ -547,7 +553,7 @@ class Config(_Config):
         else:
             return bool(_SHEBANG_RE.match(line))
 
-    def _check_folder_gitignore(self, folder: str) -> Optional[Path]:
+    def _check_folder_git_ls_files(self, folder: str) -> Optional[Path]:
         env = {**os.environ, "LANG": "C.UTF-8"}
         try:
             topfolder_result = subprocess.check_output(  # nosec # skipcq: PYL-W1510
@@ -558,26 +564,30 @@ class Config(_Config):
 
         git_folder = Path(topfolder_result.rstrip()).resolve()
 
-        files: List[str] = []
-        # don't check symlinks; either part of the repo and would be checked
-        # twice, or is external to the repo and git won't know anything about it
-        for root, _dirs, git_files in os.walk(git_folder, followlinks=False):
-            if ".git" in _dirs:
-                _dirs.remove(".git")
-            for git_file in git_files:
-                files.append(os.path.join(root, git_file))
-        git_options = ["-C", str(git_folder), "-c", "core.quotePath="]
-        try:
-            ignored = subprocess.check_output(  # nosec # skipcq: PYL-W1510
-                ["git", *git_options, "check-ignore", "-z", "--stdin", "--no-index"],
+        # files committed to git
+        tracked_files = (
+            subprocess.check_output(  # nosec # skipcq: PYL-W1510
+                ["git", "-C", str(git_folder), "ls-files", "-z"],
                 encoding="utf-8",
                 env=env,
-                input="\0".join(files),
             )
-        except subprocess.CalledProcessError:
-            return None
+            .rstrip("\0")
+            .split("\0")
+        )
+        # files that haven't been committed yet, but aren't ignored
+        tracked_files_others = (
+            subprocess.check_output(  # nosec # skipcq: PYL-W1510
+                ["git", "-C", str(git_folder), "ls-files", "-z", "--others", "--exclude-standard"],
+                encoding="utf-8",
+                env=env,
+            )
+            .rstrip("\0")
+            .split("\0")
+        )
 
-        self.git_ignore[git_folder] = {Path(f) for f in ignored.rstrip("\0").split("\0")}
+        self.git_ls_files[git_folder] = {
+            str(git_folder / Path(f)) for f in tracked_files + tracked_files_others
+        }
         return git_folder
 
     def is_skipped(self, file_path: Path) -> bool:
@@ -619,14 +629,20 @@ class Config(_Config):
             git_folder = None
 
             file_paths = [file_path, file_path.resolve()]
-            for folder in self.git_ignore:
+            for folder in self.git_ls_files:
                 if any(folder in path.parents for path in file_paths):
                     git_folder = folder
                     break
             else:
-                git_folder = self._check_folder_gitignore(str(file_path.parent))
+                git_folder = self._check_folder_git_ls_files(str(file_path.parent))
 
-            if git_folder and any(path in self.git_ignore[git_folder] for path in file_paths):
+            # git_ls_files are good files you should parse. If you're not in the allow list, skip.
+
+            if (
+                git_folder
+                and not file_path.is_dir()
+                and str(file_path.resolve()) not in self.git_ls_files[git_folder]
+            ):
                 return True
 
         return False
