@@ -12,7 +12,7 @@ from isort import Config, exceptions
 
 def test_semicolon_ignored_for_dynamic_lines_after_import_issue_1178():
     """Test to ensure even if a semicolon is in the decorator in the line following an import
-    the correct line spacing detrmination will be made.
+    the correct line spacing determination will be made.
     See: https://github.com/pycqa/isort/issues/1178.
     """
     assert isort.check_code(
@@ -617,7 +617,7 @@ quiet = true
 quiet = true
 """
     )
-    with pytest.warns(None) as warning:
+    with pytest.warns(None) as warning:  # type: ignore
         assert Config(settings_file=str(settings_file)).quiet
     assert not warning
 
@@ -1056,3 +1056,20 @@ def test_sort_configurable_sort_issue_1732() -> None:
     )
     with pytest.raises(exceptions.SortingFunctionDoesNotExist):
         isort.code(test_input, sort_order="round")
+
+
+def test_cython_pure_python_imports_2062():
+    """Test to ensure an import form a cython.cimports remains import, not cimport.
+    See: https://github.com/pycqa/isort/issues/2062.
+    """
+    assert isort.check_code(
+        """
+import cython
+from cython.cimports.libc import math
+
+
+def use_libc_math():
+    return math.ceil(5.5)
+""",
+        show_diff=True,
+    )
